(*This file does the calculations for Lemma C121.*)


    
 (**************INTERACTION ENERGIES***********************)  
ST[{x_,y_,z_}]:={x/(1-z),y/(1-z)}
IST[{x_,y_}]:={2 x/(1+x x + y y),2 y/(1+x x + y y),1-2/(1+x x + y y)}
DIST2[v_,w_]:=(v-w).(v-w);
ENERGY2A[p_,s_]:=Power[1/DIST2[IST[p],{0,0,1}],s/2];
ENERGY2[p_,q_,s_]:= Power[1/DIST2[IST[p],IST[q]],s/2];
(************************************************************)




(***********COMPUTING THE BASIC FUNCTIONS***************************)
(*This is the parameterization*)
f[t_]:=(55 (1-t) + 56 t)/64;

(*This is the function H from the proof*)
Module[{point0,point1,point2,point3},H1[a_,b_,s_]:=(
		 point0={+f[a],0};
                 point1={0,-f[b]};
		 point2={-f[a],0};
		 point3={0,+f[b]};
                 2 ENERGY2A[point0,s]+ 2 ENERGY2A[point1,s] +  4 ENERGY2[point0,point1,s])]

(*This is the composition of H with sigma, the symmetrization operation*)
Module[{u,v,t,point0,point1,point2,point3},H2[a_,b_,s_]:=(  
                 u=f[a];
                 v=f[b];
                 t=(u+v +  (u-v) (u-v))/2;
		 point0={+t,0};
                 point1={0,-t};
		 point2={-t,0};
		 point3={0,+t};
                 2 ENERGY2A[point0,s]+ 2 ENERGY2A[point1,s] + 4 ENERGY2[point0,point1,s])];

  B[a_,b_,s_]:=H1[a,b,s]-H2[a,b,s];

  
(*In the first polynomial the minus sign is deliberate*)
poly2=Expand[Numerator[Together[Expand[B[a,b,2]]]]];
poly2=Expand[Factor[-poly2/(a-b)/(a-b)]]

poly14=Expand[Numerator[Together[Expand[B[a,b,14]]]]];
poly14=Expand[Factor[poly14/(a-b)/(a-b)]]

poly16=Expand[Numerator[Together[Expand[B[a,b,16]]]]];
poly16=Expand[Factor[poly16/(a-b)/(a-b)]]
  
(*************************END POLYNOMIALS**************************)


  
(**********5 VARIABLE POSDOM TEST*********************************)
  (*We only need the 2 variable case here*)
  

Module[{TRANS5,MAX1,MAX2,MAX3,MAX4,MAX5,coeffs,monomials,exponents,LIST},Posdom5[poly_]:=(
		 coeffs=Table[poly[[j]]/.{a->1,b->1,c->1,d->1,e->1},{j,1,Length[poly]}];
                 monomials=Table[poly[[j]]/coeffs[[j]],{j,1,Length[poly]}];
                 exponents=Table[ExtractExponent5[monomials[[j]]],{j,1,Length[poly]}];

                 (*getting the maximum exponents*)
                 TRANS5=Transpose[exponents];
                 MAX1=Max[TRANS5[[1]]];
                 MAX2=Max[TRANS5[[2]]];
                 MAX3=Max[TRANS5[[3]]];
                 MAX4=Max[TRANS5[[4]]];
                 MAX5=Max[TRANS5[[5]]];
Print["      exponent limits ",MAX1," ",MAX2," ",MAX3," ",MAX4," ",MAX5]; 
                 (*done getting maximum exponents*)

		 (*tailored to the one polynomial we use it on*)
LIST=Table[Table[Table[Table[Table[SumCoeffs[{i1,i2,i3,i4,i5},exponents,coeffs],{i1,0,MAX1}],{i2,0,MAX2}],{i3,0,MAX3}],{i4,0,MAX4}],{i5,0,MAX5}];
                 {Min[Flatten[LIST]],Max[Flatten[LIST]]})];



                 (*supporting routines*)

SumCoeffs[BOUND_,exponents_,coeffs_]:=Sum[SingleTerm[BOUND,exponents,coeffs,k],{k,1,Length[coeffs]}]
SingleTerm[BOUND_,exponents_,coeffs_,k_]:=ExponentDominated[BOUND,exponents[[k]]] coeffs[[k]];
ExponentDominated[BOUND_,small_]:=If[Min[BOUND-small]<0,0,1];
                 

(*extracts the exponent.*)

Module[{val,fac,P},ExtractExponent5[mono_]:=(
		  P=Table[Prime[j],{j,100,105}];
                  val=mono/.{a->P[[1]],b->P[[2]],c->P[[3]],d->P[[4]],e->P[[5]]};
                  fac=FactorInteger[val];
                  Table[ex5[fac,P[[j]]],{j,1,5}])];

Module[{pos},ex5[fac_,p_]:=(
              pos=Position[fac,p];
              If[Length[pos]==0,0,fac[[pos[[1,1]],2]]])];
            
(**************END POSDOM5  METHOD***********************************)

       
       
       
(*********************FINAL TESTS: takes less than an hour***********************)
  Print[Posdom5[poly2]];
  Print[Posdom5[poly14]];
  Print[Posdom5[poly16]];
 
